/**
 *    Python.java
 *    Copyright (C) 2010 New Zealand Digital Library, http://expeditee.org
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.expeditee.actions;

import java.awt.Color;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import javax.script.Bindings;
import javax.script.Invocable;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.script.SimpleScriptContext;

import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.FrameUtils;
import org.expeditee.gui.MessageBay;
import org.expeditee.io.flowlayout.XGroupItem;
import org.expeditee.items.Item;
import org.expeditee.items.ItemUtils;
import org.expeditee.items.Text;

/**
 * Javascript parser.
 * Works differently to the other Javascript class in that it
 * parses a frame as a whole rather than parsing individual text items as separate statements
 * 
 * @author jts21
 * @author davidb
 */
public class Python extends ScriptBase {
	
    protected void init() {
	ERROR_FRAMESET = "PythonErrors";
	scriptEngineManager = new ScriptEngineManager();
	scriptEngine = scriptEngineManager.getEngineByMimeType("application/python");
	scriptEngine.put("invocable", (Invocable) scriptEngine);
    }
    
    public static void printPythonFrame(Item item) {
	if(item.getChild() == null) {
	    // if the user clicked on the action without an item on their cursor
	    if(item.hasAction()) {
		boolean isThis = false;
		for(String s : item.getAction()) {
		    if(s.equalsIgnoreCase("printPythonFrame")) {
			isThis = true;
			break;
		    }
		}
		if(isThis) {
		    System.out.println(new Python(item.getParentOrCurrentFrame(), true));
		    return;
		}
	    }
	    MessageBay.warningMessage("Requires either an item with a link to a frame, or no item (will run the current frame)");
	} else {
	    System.out.println(new Python(item.getChild(), true));
	}
    }
    
    public static void runPythonFrame(Item item) throws Exception {
	if(item.getChild() == null) {
	    // if the user clicked on the action without an item on their cursor
	    if(item.hasAction()) {
		boolean isThis = false;
		for(String s : item.getAction()) {
		    if(s.equalsIgnoreCase("runPythonFrame")) {
			isThis = true;
			break;
		    }
		}
		if(isThis) {
		    Python.runFrameFollow(item.getParentOrCurrentFrame());
		    return;
		}
	    }
	    MessageBay.warningMessage("Requires either an item with a link to a frame, or no item (will run the current frame)");
	} else {
	    Python.runFrameFollow(item.getChild());
	}
    }
    
    
    private static synchronized void runFrameFollow(Frame frame) throws Exception {
	Python python = new Python(frame, true);
	python.runFrame(frame,true);   
    }
    
    private Python(Frame frame, boolean followLinks) {
	super(frame,followLinks);
    }
    
    
    protected void processItems(List<Item> overlapping_y_ordered_items, boolean followLinks, int depth) {
	
	// Loop through the items looking for code and links to new frames
	for(Item i : overlapping_y_ordered_items) {
	    if(followLinks && i.hasLink()) {
		Frame child = i.getChild();
		if(child != null && !seen.contains(child)) {
		    this.parseFrame(child, true, depth);
		}
	    }
	    if(i instanceof Text && !i.isAnnotation()) {
		String text = ((Text)i).getText();
		if (i == org.expeditee.io.flowlayout.XGroupItem.GROUPSEP_START) {
		    text = null;
		    depth++;
		}
		else if (i == org.expeditee.io.flowlayout.XGroupItem.GROUPSEP_END) {
		    text = null;
		    depth--;
		}

		if (text != null) {
		    int lineNumber = 0;
		    for(String line : text.trim().split("[\\n\\r]+")) {
			if (depth>0) {
			    sb.append(String.format("%1$" + (4*depth) + "s", "")); // indent with 'depth' x 4 spaces 	
			}
			sb.append(line).append("\n");
			lines.add(new CodeLine((Text)i, lineNumber++, line));
		    }
		}
	    }
	}

	
    }

}
